==> choreonoid_1.5.0+dfsg+20150714+337+15~ubuntu14.10.1.tar.xz: choreonoid-1.5.0+dfsg+20150714+337+15~ubuntu14.10.1/src/Body/SceneBody.h <==
/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_SCENE_BODY_H
#define CNOID_BODY_SCENE_BODY_H

#include "Body.h"
#include "SceneDevice.h"
#include <cnoid/SceneGraph>
#include "exportdecl.h"

namespace cnoid {

class CNOID_EXPORT SceneLink : public SgPosTransform
{
public:
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW;
        
    SceneLink(Link* link);
    Link* link() { return link_; }
    const Link* link() const { return link_; }
    const SgNode* shape() const { return shape_; }
    SgNode* shape() { return shape_; }
    const SgNode* orgShape() const { return orgShape_; }
    void cloneShape(SgCloneMap& cloneMap);
    void addSceneDevice(SceneDevice* sdev);
    SceneDevice* getSceneDevice(Device* device);
    bool isVisible() const;
    void setVisible(bool on);
    void makeTransparent(float transparency);
    void makeTransparent(float transparency, SgCloneMap& cloneMap);

private:
    SceneLink(const SceneLink& org);
    Link* link_;
    SgNodePtr orgShape_;
    SgNodePtr shape_;
    std::vector<SceneDevicePtr> sceneDevices_;
    SgGroupPtr deviceGroup;
    float transparency_;
};
typedef ref_ptr<SceneLink> SceneLinkPtr;
    
    
class CNOID_EXPORT SceneBody : public SgPosTransform
{
public:
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW;
        
    SceneBody(BodyPtr body);
    SceneBody(BodyPtr body, boost::function<SceneLink*(Link*)> sceneLinkFactory);

    Body* body() { return body_; }
    const Body* body() const { return body_; }

    void cloneShapes(SgCloneMap& cloneMap);

    int numSceneLinks() const { return sceneLinks_.size(); }
    SceneLink* sceneLink(int index) { return sceneLinks_[index]; }
    const SceneLink* sceneLink(int index) const { return sceneLinks_[index]; }

    void updateLinkPositions();
    void updateLinkPositions(SgUpdate& update);

    SceneDevice* getSceneDevice(Device* device);
    void setSceneDeviceUpdateConnection(bool on);
    void updateSceneDevices();

    void makeTransparent(float transparency);
    void makeTransparent(float transparency, SgCloneMap& cloneMap);

protected:
    BodyPtr body_;
    std::vector<SceneLinkPtr> sceneLinks_;
    std::vector<SceneDevicePtr> sceneDevices;

    virtual ~SceneBody();

private:
    SceneBody(const SceneBody& org);
    void initialize(BodyPtr& body, const boost::function<SceneLink*(Link*)>& sceneLinkFactory);
};
            
typedef ref_ptr<SceneBody> SceneBodyPtr;
}
    
#endif

==> choreonoid_1.6.0+dfsg+20180503+1250+23~ubuntu16.04.1.tar.xz: choreonoid-1.6.0+dfsg+20180503+1250+23~ubuntu16.04.1/src/Body/SceneBody.h <==
/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_SCENE_BODY_H
#define CNOID_BODY_SCENE_BODY_H

#include "Body.h"
#include "SceneDevice.h"
#include <cnoid/SceneGraph>
#include "exportdecl.h"

namespace cnoid {

class SceneLinkImpl;

class CNOID_EXPORT SceneLink : public SgPosTransform
{
public:
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
        
    SceneLink(Link* link);
    ~SceneLink();

    Link* link() { return link_; }
    const Link* link() const { return link_; }

    const SgNode* visualShape() const;
    SgNode* visualShape();
    const SgNode* collisionShape() const;
    SgNode* collisionShape();

    void insertEffectGroup(SgGroup* group);
    void removeEffectGroup(SgGroup* group);
    
    void setVisible(bool on);
    void makeTransparent(float transparency);
    
    void addSceneDevice(SceneDevice* sdev);
    SceneDevice* getSceneDevice(Device* device);

private:
    SceneLink(const SceneLink& org);

    Link* link_;
    SceneLinkImpl* impl;
    friend class SceneBody;
};

typedef ref_ptr<SceneLink> SceneLinkPtr;

class SceneBodyImpl;
    
class CNOID_EXPORT SceneBody : public SgPosTransform
{
public:
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
        
    SceneBody(Body* body);
    SceneBody(Body* body, std::function<SceneLink*(Link*)> sceneLinkFactory);
    virtual ~SceneBody();

    Body* body() { return body_; }
    const Body* body() const { return body_; }

    void cloneShapes(SgCloneMap& cloneMap);

    int numSceneLinks() const { return sceneLinks_.size(); }
    SceneLink* sceneLink(int index) { return sceneLinks_[index]; }
    const SceneLink* sceneLink(int index) const { return sceneLinks_[index]; }

    void updateLinkPositions();
    void updateLinkPositions(SgUpdate& update);

    SceneDevice* getSceneDevice(Device* device);
    void setSceneDeviceUpdateConnection(bool on);
    void updateSceneDevices(double time);

    void makeTransparent(float transparency);
    void makeTransparent(float transparency, SgCloneMap& cloneMap);

    virtual void updateModel();

private:
    SceneBody(const SceneBody& org);

    BodyPtr body_;
    std::vector<SceneLinkPtr> sceneLinks_;
    SceneBodyImpl* impl;
};
            
typedef ref_ptr<SceneBody> SceneBodyPtr;

}
    
#endif
